@@ -57,9 +57,9 @@ static GstElementStateReturn
 static const GstFormat* gst_flacdec_get_src_formats 	(GstPad *pad);
 static gboolean 	gst_flacdec_convert_src	 	(GstPad *pad, GstFormat src_format, gint64 src_value,
 		        				 GstFormat *dest_format, gint64 *dest_value);
-static const GstPadQueryType* 
+static const GstQueryType* 
 			gst_flacdec_get_src_query_types (GstPad *pad);
-static gboolean		gst_flacdec_src_query 		(GstPad *pad, GstPadQueryType type,
+static gboolean		gst_flacdec_src_query 		(GstPad *pad, GstQueryType type,
 	               					 GstFormat *format, gint64 *value);
 static const GstEventMask* 
 			gst_flacdec_get_src_event_masks (GstPad *pad);
@@ -220,7 +220,7 @@ gst_flacdec_seek (const FLAC__SeekableStreamDecoder *decoder,
 
   flacdec = GST_FLACDEC (client_data);
 
-  GST_DEBUG (0, "seek %lld\n", position);
+  GST_DEBUG (0, "seek %lld", position);
   if (!gst_bytestream_seek (flacdec->bs, position, GST_SEEK_METHOD_SET)) {
     return FLAC__SEEKABLE_STREAM_DECODER_SEEK_STATUS_ERROR;
   }
@@ -239,7 +239,7 @@ gst_flacdec_tell (const FLAC__SeekableStreamDecoder *decoder,
   if (*position == -1)
     return FLAC__SEEKABLE_STREAM_DECODER_TELL_STATUS_ERROR;
 
-  GST_DEBUG (0, "tell %lld\n", *position);
+  GST_DEBUG (0, "tell %lld", *position);
 
   return FLAC__SEEKABLE_STREAM_DECODER_TELL_STATUS_OK;
 }
@@ -256,7 +256,7 @@ gst_flacdec_length (const FLAC__SeekableStreamDecoder *decoder,
   if (*length == -1)
     return FLAC__SEEKABLE_STREAM_DECODER_TELL_STATUS_ERROR;
 
-  GST_DEBUG (0, "length %lld\n", *length);
+  GST_DEBUG (0, "length %lld", *length);
 
   return FLAC__SEEKABLE_STREAM_DECODER_LENGTH_STATUS_OK;
 }
@@ -268,7 +268,7 @@ gst_flacdec_eof (const FLAC__SeekableStreamDecoder *decoder,
   FlacDec *flacdec;
 
   flacdec = GST_FLACDEC (client_data);
-  GST_DEBUG (0, "eof %d\n", flacdec->eos);
+  GST_DEBUG (0, "eof %d", flacdec->eos);
 
   return flacdec->eos;
 }
@@ -433,44 +433,50 @@ gst_flacdec_loop (GstElement *element)
 
   flacdec = GST_FLACDEC (element);
 
+  GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: entering loop");
   if (flacdec->init) {
+    GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: initializing decoder");
     FLAC__seekable_stream_decoder_init (flacdec->decoder);
-    //FLAC__seekable_stream_decoder_process_metadata (flacdec->decoder);
+    /* FLAC__seekable_stream_decoder_process_metadata (flacdec->decoder); */
     flacdec->init = FALSE;
   }
 
   if (flacdec->seek_pending) {
-    GST_DEBUG (GST_CAT_EVENT, "perform seek to sample %lld\n", 
+    GST_DEBUG (GST_CAT_EVENT, "perform seek to sample %lld", 
 		              flacdec->seek_value);
 
     if (FLAC__seekable_stream_decoder_seek_absolute (flacdec->decoder, 
 			                             flacdec->seek_value)) 
     {
       flacdec->total_samples = flacdec->seek_value;
-      GST_DEBUG (GST_CAT_EVENT, "seek done\n");
+      GST_DEBUG (GST_CAT_EVENT, "seek done");
     }
     else {
-      GST_DEBUG (GST_CAT_EVENT, "seek failed\n");
+      GST_DEBUG (GST_CAT_EVENT, "seek failed");
     }
     flacdec->seek_pending = FALSE;
   }
 
+  GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: processing single");
   res = FLAC__seekable_stream_decoder_process_single (flacdec->decoder);
+  GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: checking for EOS");
   if (FLAC__seekable_stream_decoder_get_state (flacdec->decoder) == 
 		  FLAC__SEEKABLE_STREAM_DECODER_END_OF_STREAM) 
   {
     GstEvent *event;
 
+    GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: sending EOS event");
     FLAC__seekable_stream_decoder_finish(flacdec->decoder);
     flacdec->init = TRUE;
 
-    event = gst_event_new(GST_EVENT_EOS);
+    event = gst_event_new (GST_EVENT_EOS);
     gst_pad_push (flacdec->srcpad, GST_BUFFER (event));
     gst_element_set_eos (element);
   }
+   GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: _loop end");
 }
 
-GST_FORMATS_FUNCTION (gst_flacdec_get_src_formats,
+GST_PAD_FORMATS_FUNCTION (gst_flacdec_get_src_formats,
   GST_FORMAT_UNITS,
   GST_FORMAT_BYTES,
   GST_FORMAT_TIME
@@ -546,19 +552,19 @@ gst_flacdec_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
 }
 
 GST_PAD_QUERY_TYPE_FUNCTION (gst_flacdec_get_src_query_types,
-  GST_PAD_QUERY_TOTAL,
-  GST_PAD_QUERY_POSITION
+  GST_QUERY_TOTAL,
+  GST_QUERY_POSITION
 )
 
 static gboolean
-gst_flacdec_src_query (GstPad *pad, GstPadQueryType type,
+gst_flacdec_src_query (GstPad *pad, GstQueryType type,
 	               GstFormat *format, gint64 *value)
 {
   gboolean res = TRUE;
   FlacDec *flacdec = GST_FLACDEC (gst_pad_get_parent (pad));
 
   switch (type) {
-    case GST_PAD_QUERY_TOTAL:
+    case GST_QUERY_TOTAL:
     {
       guint64 samples;
 
@@ -573,7 +579,7 @@ gst_flacdec_src_query (GstPad *pad, GstPadQueryType type,
 		       format, value);
       break;
     }
-    case GST_PAD_QUERY_POSITION:
+    case GST_QUERY_POSITION:
       gst_pad_convert (flacdec->srcpad, 
 		       GST_FORMAT_UNITS, 
 		       flacdec->total_samples, 
@@ -587,7 +593,7 @@ gst_flacdec_src_query (GstPad *pad, GstPadQueryType type,
   return res;
 }
 	  
-GST_EVENT_MASK_FUNCTION (gst_flacdec_get_src_event_masks,
+GST_PAD_EVENT_MASK_FUNCTION (gst_flacdec_get_src_event_masks,
     { GST_EVENT_SEEK, GST_SEEK_FLAG_ACCURATE }
 );
 
